import cjen

from cjen.nene.helper import FileHelper

from pyodm.core.source.abstract_database_source import AbstractDataBaseSource
from pyodm.tests.test_listable_factory.mappers.subject_mapper import SubjectMapper


class SubjectDataResource(AbstractDataBaseSource):

    def read(self) -> list[list[dict]]:
        return self.subjects()

    def __init__(self, database_info):
        super().__init__(database_info)

    @cjen.operate.mysql.factory(
        clazz=SubjectMapper,
        sql=FileHelper.read(SubjectMapper.sql_file),
        size=-1
    )
    def subjects(self, subject_mappers: list[SubjectMapper] = None, **kwargs):
        return [subject_mapper(self.registry) for subject_mapper in subject_mappers]
        # return [subject_mapper.mapper(self.registry) for subject_mapper in subject_mappers]



from cjen import MetaMysql

from pyodm.model.definition.mapper.assemble import CdiscSQL
from pyodm.utils.path_utils import PathUtils

path = PathUtils.folder("pyodm", __file__)


@CdiscSQL(
    sql_path=path.joinpath("tests", "test_listable_factory", "sqls", "subjects.sql"),
    mapper_path=path.joinpath("tests", "test_listable_factory", "mappers", "mapper.xml"))
class SubjectMapper(MetaMysql): ...



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class CDISCNotes(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/CDISCNotes
    """
    
    
    TranslatedText = Model.ManyElements()
    



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class StudyTargetPopulation(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/StudyTargetPopulation
    """
    
    OID = Model.Attribute()
    
    Name = Model.Attribute()
    
    
    Description = Model.OneElement()
    
    Coding = Model.ManyElements()
    
    FormalExpression = Model.ManyElements()
    



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class StudyParameter(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/StudyParameter
    """
    
    OID = Model.Attribute()
    
    Term = Model.Attribute()
    
    ShortName = Model.Attribute()
    
    
    ParameterValue = Model.ManyElements()
    
    Coding = Model.ManyElements()
    



from pyodm.model.meta.entity_meta import EntityMeta
import pyodm.model.definition as Model


class CdiscODMEntity(metaclass=EntityMeta):
    """
    自定义ODM的元素的时候，必须继承的元类
    """
    def __init__(self):
        self._name = ""
        self.value = None
        for name, instance in vars(self.__class__).items():
            if isinstance(instance, Model.Attribute) \
                    or isinstance(instance, Model.OneElement) \
                    or isinstance(instance, Model.ManyElements):
                setattr(self, name, instance.__class__())

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    def is_blank(self) -> bool: return self.value is None



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class SubjectData(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/SubjectData+Element
    """
    
    SubjectKey = Model.Attribute()
    
    TransactionType = Model.Attribute()
    
    
    InvestigatorRef = Model.OneElement()
    
    SiteRef = Model.OneElement()
    
    StudyEventData = Model.ManyElements()
    
    Query = Model.ManyElements()
    
    AuditRecord = Model.OneElement()
    
    Signature = Model.OneElement()
    
    Annotation = Model.ManyElements()
    



import abc

from pyodm.factory.cdisc_registry import CdiscRegistry


class AbstractRegistration(abc.ABC):
    def __init__(self):
        self._registry: CdiscRegistry = None

    @property
    def registry(self):
        if self._registry is None: raise Exception("registry is None in Registration")
        return self._registry

    @registry.setter
    def registry(self, value: CdiscRegistry): self._registry = value



class XpathNode:
    def __init__(self, name):
        self.name = name
        self.parent: list[int] = []
        self.children: list[int] = []

    def __eq__(self, other):
        return self.name == other.name

    def isRoot(self): return len(self.parent) == 0



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class ItemRef(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/ItemRef
    """
    
    ItemOID = Model.Attribute()
    
    KeySequence = Model.Attribute()
    
    IsNonStandard = Model.Attribute()
    
    HasNoData = Model.Attribute()
    
    MethodOID = Model.Attribute()
    
    UnitsItemOID = Model.Attribute()
    
    Repeat = Model.Attribute()
    
    Other = Model.Attribute()
    
    Role = Model.Attribute()
    
    RoleCodeListOID = Model.Attribute()
    
    Core = Model.Attribute()
    
    PreSpecifiedValue = Model.Attribute()
    
    OrderNumber = Model.Attribute()
    
    Mandatory = Model.Attribute()
    
    CollectionExceptionConditionOID = Model.Attribute()
    
    
    Origin = Model.ManyElements()
    
    WhereClauseRef = Model.ManyElements()
    



from pyodm.core.support.abstract_path_loader import AbstractPathDataLoader

from lxml import etree


class XmlPathLoader(AbstractPathDataLoader):
    def load(self):
        return etree.parse(self.file)



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class User(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/User+Element
    """
    
    OID = Model.Attribute()
    
    UserType = Model.Attribute()
    
    OrganizationOID = Model.Attribute()
    
    LocationOID = Model.Attribute()
    
    
    UserName = Model.OneElement()
    
    Prefix = Model.OneElement()
    
    Suffix = Model.OneElement()
    
    FullName = Model.OneElement()
    
    GivenName = Model.OneElement()
    
    FamilyName = Model.OneElement()
    
    Image = Model.OneElement()
    
    Address = Model.ManyElements()
    
    Telecom = Model.ManyElements()
    



import pathlib
from abc import ABC

from pyodm.core.data_loader.xml_path_loader import XmlPathLoader
from pyodm.core.source.path_source import PathSource
from pyodm.core.xml.reader.data.xml_data_reader import XMLDataReader
from pyodm.factory.abstract_cdisc_factory import AbstractCdiscFactory
from pyodm.factory.abstract_hierarchy_factory import AbstractHierarchyFactory


class AbstractCdiscXMLFactory(AbstractHierarchyFactory, ABC):
    """
    通过 XML 生成ODM的工程对象的类型，定义了生成模板
        1. 获取 Class 类型 class_reader
        2. 读取 XML 文件转化为ODM 对象 data_reader
    """

    def __init__(self, data_file_path: PathSource, configuration_files: list = None):
        super().__init__(configuration_files)
        self._data_file = data_file_path.read()
        self.data_loader = XmlPathLoader(self._data_file)

    def data_reader(self) -> XMLDataReader:
        """
        指定了解析 ODM XML Data 的数据方式
        :return:
        :rtype:
        """
        return XMLDataReader(self)





from lxml import etree
from lxml.etree import Element

from pyodm.exceptions import AttributeException, ErrCode


class EtreeUtils:
    @staticmethod
    def localname(element):
        return etree.QName(element.tag).localname

    @staticmethod
    def attrib(element: Element, attrib_name, strict=False):
        value = element.attrib.get(attrib_name)
        if not strict: return value
        if value is None: raise AttributeException(attrib_name, EtreeUtils.localname(element), ErrCode.NoExist)
        return value



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class SourceItems(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/SourceItems
    """
    
    
    SourceItem = Model.ManyElements()
    
    Coding = Model.ManyElements()
    



import pytest

from pyodm.utils.forest import Forest


@pytest.fixture
def data_one():
    return [dict(site_id=1, site_name="D01")]


@pytest.fixture
def tree_array():
    return [
        [dict(id=1), dict(id=2), dict(id=5)],
        [dict(id=1), dict(id=3)],
        [dict(id=1), dict(id=4)]
    ]


def test_transform(tree_array):
    tree = Forest.transform(tree_array)
    assert len(tree.nodes) == 3
    assert tree.degree == 3
    root = tree.root()
    assert root.get("id") == 1

    assert len(root.children()) == 3
    assert root.children()[0]._get("id") == 2
    assert root.children()[1]._get("id") == 3
    assert root.children()[2]._get("id") == 4

    assert root.children()[0].has_child() is True
    assert root.children()[1].has_child() is False
    assert root.children()[2].has_child() is False

    assert root.children()[0].children()[0]._get("id") == 5


def test_one_level_tree(data_one):
    tree = Forest.factory(branch=data_one)
    assert len(tree.nodes) == 1
    assert len(tree.get_level_nodes(0)) == 1
    root0 = tree.get_level_nodes(0)[0]
    assert root0.values["site_id"] == 1
    assert root0.values["site_name"] == "D01"
    tree.add_branch(data_one)
    assert len(tree.nodes) == 1
    assert len(tree.get_level_nodes(0)) == 1
    assert root0.values["site_id"] == 1
    assert root0.values["site_name"] == "D01"


@pytest.fixture
def data_two():
    return [[dict(site_id=1, site_name="D01"), dict(subject_id=1, subject_name="S01")],
            [dict(site_id=1, site_name="D01"), dict(subject_id=1, subject_name="S01")],
            [dict(site_id=1, site_name="D01"), dict(subject_id=2, subject_name="S02")],
            [dict(site_id=2, site_name="D02"), dict(subject_id=1, subject_name="S01")],
            [dict(site_id=2, site_name="D02"), dict(subject_id=2, subject_name="S02")],
            [dict(site_id=2, site_name="D02"), dict(subject_id=3, subject_name="S04")],
            [dict(site_id=2, site_name="D03"), dict(subject_id=3, subject_name="S04")]]


def test_two_level_tree(data_two):
    tree = Forest.factory()
    for e in data_two:
        tree.add_branch(e)
        assert len(tree.nodes) == 2

    assert len(tree.get_level_nodes(0)) == 3
    root0 = tree.get_level_nodes(0)[0]
    root1 = tree.get_level_nodes(0)[1]
    root2 = tree.get_level_nodes(0)[2]
    assert root0.values["site_id"] == 1
    assert root0.values["site_name"] == "D01"
    assert root1.values["site_id"] == 2
    assert root1.values["site_name"] == "D02"
    assert root2.values["site_id"] == 2
    assert root2.values["site_name"] == "D03"
    assert len(tree.get_level_nodes(1)) == 6
    assert len(root0.children_index) == 2
    assert root0.find_child(subject_id=1, subject_name="S01") is not None
    assert root0.find_child(subject_id=1, subject_name="S01").parent().satisfy(site_id=1, site_name="D01") is True
    assert root0.find_child(subject_id=2, subject_name="S02") is not None
    assert root0.find_child(subject_id=2, subject_name="S02").parent().satisfy(site_id=1, site_name="D01") is True
    assert len(root1.children_index) == 3
    assert root1.find_child(subject_id=1, subject_name="S01") is not None
    assert root1.find_child(subject_id=1, subject_name="S01").parent().satisfy(site_id=2, site_name="D02") is True
    assert root1.find_child(subject_id=2, subject_name="S02") is not None
    assert root1.find_child(subject_id=2, subject_name="S02").parent().satisfy(site_id=2, site_name="D02") is True
    assert root1.find_child(subject_id=3, subject_name="S04") is not None
    assert root1.find_child(subject_id=3, subject_name="S04").parent().satisfy(site_id=2, site_name="D02") is True
    assert len(root2.children_index) == 1
    assert root2.find_child(subject_id=3, subject_name="S04") is not None
    assert root2.find_child(subject_id=3, subject_name="S04").parent().satisfy(site_id=2, site_name="D03") is True


@pytest.fixture
def data_three():
    return [
        [dict(site_id=1, site_name="D01"), dict(subject_id=1, subject_name="S01"), dict(visit_id=1, visit_name="V01")],
        [dict(site_id=1, site_name="D01"), dict(subject_id=1, subject_name="S01"), dict(visit_id=1, visit_name="V01")],
        [dict(site_id=1, site_name="D01"), dict(subject_id=1, subject_name="S01"), dict(visit_id=2, visit_name="V02")],
        [dict(site_id=1, site_name="D01"), dict(subject_id=2, subject_name="S02"), dict(visit_id=1, visit_name="V01")],
        [dict(site_id=2, site_name="D02"), dict(subject_id=1, subject_name="S01"), dict(visit_id=1, visit_name="V01")],
        [dict(site_id=2, site_name="D02"), dict(subject_id=2, subject_name="S02"), dict(visit_id=2, visit_name="V02")],
        [dict(site_id=2, site_name="D02"), dict(subject_id=3, subject_name="S04"), dict(visit_id=3, visit_name="V03")],
        [dict(site_id=2, site_name="D03"), dict(subject_id=3, subject_name="S04"), dict(visit_id=1, visit_name="V01")]]


def test_three_level_tree(data_three):
    tree = Forest.factory()
    for e in data_three:
        tree.add_branch(e)
    assert len(tree.nodes) == 3
    tree.reset(data_three)
    assert len(tree.nodes) == 3
    assert len(tree.get_level_nodes(0)) == 3
    root0 = tree.get_level_nodes(0)[0]
    root1 = tree.get_level_nodes(0)[1]
    root2 = tree.get_level_nodes(0)[2]
    assert root0.values["site_id"] == 1
    assert root0.values["site_name"] == "D01"
    assert root1.values["site_id"] == 2
    assert root1.values["site_name"] == "D02"
    assert root2.values["site_id"] == 2
    assert root2.values["site_name"] == "D03"
    assert len(tree.get_level_nodes(1)) == 6
    assert len(root0.children_index) == 2
    assert root0.find_child(subject_id=1, subject_name="S01") is not None
    assert root0.find_child(subject_id=1, subject_name="S01").parent().satisfy(site_id=1, site_name="D01") is True
    assert root0.find_child(subject_id=2, subject_name="S02") is not None
    assert root0.find_child(subject_id=2, subject_name="S02").parent().satisfy(site_id=1, site_name="D01") is True
    assert len(root1.children_index) == 3
    assert root1.find_child(subject_id=1, subject_name="S01") is not None
    assert root1.find_child(subject_id=1, subject_name="S01").parent().satisfy(site_id=2, site_name="D02") is True
    assert root1.find_child(subject_id=2, subject_name="S02") is not None
    assert root1.find_child(subject_id=2, subject_name="S02").parent().satisfy(site_id=2, site_name="D02") is True
    assert root1.find_child(subject_id=3, subject_name="S04") is not None
    assert root1.find_child(subject_id=3, subject_name="S04").parent().satisfy(site_id=2, site_name="D02") is True
    assert len(root2.children_index) == 1
    assert root2.find_child(subject_id=3, subject_name="S04") is not None
    assert root2.find_child(subject_id=3, subject_name="S04").parent().satisfy(site_id=2, site_name="D03") is True

    assert len(tree.get_level_nodes(2)) == 7
    assert len(root0.find_child(subject_id=1, subject_name="S01").children_index) == 2
    assert root0.find_child(subject_id=1, subject_name="S01").find_child(visit_id=1,
                                                                         visit_name="V01").parent().parent().satisfy(
        site_id=1, site_name="D01") is True
    assert root0.find_child(subject_id=1, subject_name="S01").find_child(visit_id=2,
                                                                         visit_name="V02").parent().parent().satisfy(
        site_id=1, site_name="D01") is True
    assert len(root0.find_child(subject_id=2, subject_name="S02").children_index) == 1
    assert root0.find_child(subject_id=2, subject_name="S02").find_child(visit_id=1,
                                                                         visit_name="V01").parent().parent().satisfy(
        site_id=1, site_name="D01") is True
    assert len(root1.find_child(subject_id=1, subject_name="S01").children_index) == 1
    assert root1.find_child(subject_id=1, subject_name="S01").find_child(visit_id=1,
                                                                         visit_name="V01").parent().parent().satisfy(
        site_id=2, site_name="D02") is True
    assert len(root1.find_child(subject_id=2, subject_name="S02").children_index) == 1
    assert root1.find_child(subject_id=2, subject_name="S02").find_child(visit_id=2,
                                                                         visit_name="V02").parent().parent().satisfy(
        site_id=2, site_name="D02") is True
    assert len(root1.find_child(subject_id=3, subject_name="S04").children_index) == 1
    assert root1.find_child(subject_id=3, subject_name="S04").find_child(visit_id=3,
                                                                         visit_name="V03").parent().parent().satisfy(
        site_id=2, site_name="D02") is True
    assert len(root2.find_child(subject_id=3, subject_name="S04").children_index) == 1
    assert root2.find_child(subject_id=3, subject_name="S04").find_child(visit_id=1,
                                                                         visit_name="V01").parent().parent().satisfy(
        site_id=2, site_name="D03") is True


@pytest.fixture
def test_dict_data():
    return [
        [dict(SubjectData=dict(SubjectKey="001")), dict(StudyEventData=dict(StudyEventOID="StudyEventOID001"))],
        [dict(SubjectData=dict(SubjectKey="001")), dict(StudyEventData=dict(StudyEventOID="StudyEventOID002"))],
        [dict(SubjectData=dict(SubjectKey="002")), dict(StudyEventData=dict(StudyEventOID="StudyEventOID002"))],
        [dict(SubjectData=dict(SubjectKey="002")), dict(StudyEventData=dict(StudyEventOID="StudyEventOID002"))]
    ]


def test_dict(test_dict_data):
    tree = Forest.factory()
    for e in test_dict_data:
        tree.add_branch(e)
    assert len(tree.nodes) == 2
    subject001 = tree.get_level_nodes(0)[0]
    study_event = subject001.find_child(StudyEventData=dict(StudyEventOID="StudyEventOID001"))
    print(study_event)
    assert dict(a=dict(b=1, c=1)) == dict(a=dict(b=1, c=1))


class Xpath:
    def __init__(self, paths: list):
        self._paths = paths

    def start(self, name: str):
        self.contain(name)
        self._paths = [path[path.index(name):len(path)] for path in self._paths]
        return self

    def contain(self, name: str):
        self._paths = list(filter(lambda x: name in x, self._paths))
        return self

    def end(self, name: str):
        self.contain(name)
        self._paths = [path[0:path.index(name)+1] for path in self._paths]
        return self

    @property
    def paths(self): return self._paths



import abc

from pyodm.core.support.abstract_data_reader import AbstractDataReader
from pyodm.core.support.abstract_data_loader import AbstractDataLoader
from pyodm.factory.cdisc_registry import CdiscRegistry
from pyodm.model import odm_xsd_description


class AbstractCdiscFactory(abc.ABC, CdiscRegistry):
    """
    生成 ODM 的抽象工厂,定义类生成ODM实例的模板方法

    """

    def __init__(self):
        super().__init__()
        self._cdisc_odm = None
        self._data_loader = None

    def default_description_files(self) -> list:
        """
        默认的 XSD 文件路径 , 如果没有指定
        clazz_reader,默认采取的是XSD 的方式解析 ODM Class
        :return:
        :rtype:
        """
        return odm_xsd_description()

    @abc.abstractmethod
    def clazz_reader(self):
        """
        指定 生成 ODM Class 的方式
        :return:
        :rtype:
        """
        ...

    @abc.abstractmethod
    def data_reader(self) -> AbstractDataReader:
        """
        指定 解析 ODM Data 的方式
        :return:
        :rtype:
        """
        ...

    @property
    def data_loader(self):
        """
        读取 ODM 数据
        :return:
        :rtype:
        """
        return self._data_loader

    @data_loader.setter
    def data_loader(self, data_loader: AbstractDataLoader):
        self._data_loader = data_loader

    def _odm_process(self, refresh: bool = False):
        """
        解析 ODM Data 模板方法
        :param refresh: 是否重新读取 True Y, False N
        :type refresh:
        :return:
        :rtype:
        """
        __new = True if self._cdisc_odm is None else refresh
        if __new:
            self.clazz_reader()
            self._cdisc_odm = self.data_reader().read(self.data_loader)
        return self._cdisc_odm

    def odm(self, refresh: bool = False):
        """
        返回 ODM 对象
        :param refresh:
        :type refresh:
        :return:
        :rtype:
        """
        return self._odm_process(refresh)



import pathlib


class PathUtils:
    @staticmethod
    def folder(name: str, current: str) -> pathlib.Path:
        """
        从当前文件路径获取期望开始的绝对路径
        :param name: 期望开始的文件夹名
        :type name: str
        :param current: __file__
        :type current: str
        :return: Path
        :rtype: pathlib.Path
        """
        path = pathlib.Path(current).parent
        while path.name != name:
            path = path.parent
        return path



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class MetaDataVersion(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/MetaDataVersion
    """
    
    OID = Model.Attribute()
    
    Name = Model.Attribute()
    
    CommentOID = Model.Attribute()
    
    
    Description = Model.OneElement()
    
    Include = Model.ManyElements()
    
    Standards = Model.ManyElements()
    
    AnnotatedCRF = Model.ManyElements()
    
    SupplementalDoc = Model.ManyElements()
    
    ValueListDef = Model.ManyElements()
    
    WhereClauseDef = Model.ManyElements()
    
    Protocol = Model.ManyElements()
    
    WorkflowDef = Model.ManyElements()
    
    StudyEventGroupDef = Model.ManyElements()
    
    StudyEventDef = Model.ManyElements()
    
    ItemGroupDef = Model.ManyElements()
    
    ItemDef = Model.ManyElements()
    
    CodeList = Model.ManyElements()
    
    ConditionDef = Model.ManyElements()
    
    MethodDef = Model.ManyElements()
    
    CommentDef = Model.ManyElements()
    
    Leaf = Model.ManyElements()
    



import pathlib
from abc import ABC

from pyodm.core.support.abstract_data_loader import AbstractDataLoader


class AbstractPathDataLoader(AbstractDataLoader, ABC):
    def __init__(self, file: pathlib.Path):
        self.file = file



from pyodm.model.definition.cdisc_model import CdiscModel


class Attribute(CdiscModel):
    """
    ODM 标准中对应的 Element 的 Attribute属性
    """
    def __init__(self):
        super().__init__()
        self._value = ""

    @property
    def name(self): return self._name

    @name.setter
    def name(self, name): self._name = name

    @property
    def value(self): return self._value

    @value.setter
    def value(self, value): self._value = value


import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class Alias(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/Alias
    """
    
    Context = Model.Attribute()
    
    Name = Model.Attribute()
    
    



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class CodeListItem(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/CodeListItem
    """
    
    CodedValue = Model.Attribute()
    
    Rank = Model.Attribute()
    
    Other = Model.Attribute()
    
    OrderNumber = Model.Attribute()
    
    ExtendedValue = Model.Attribute()
    
    CommentOID = Model.Attribute()
    
    
    Description = Model.OneElement()
    
    Decode = Model.OneElement()
    
    Coding = Model.ManyElements()
    
    Alias = Model.ManyElements()
    



from pyodm.model.definition.usage import Usage


class CdiscModel:
    def __init__(self):
        self._preset_usage: Usage = None
        self._name = "NoUse!"
        self._parent = None

    def find(self, **attributes):
        raise Exception(f"{self.name} is not ManyElements, Only ManyElements have")

    @property
    def preset_usage(self):
        """
        获取预置的 业务处理
        :return:
        :rtype:
        """
        return self._preset_usage

    @preset_usage.setter
    def preset_usage(self, value):
        """
        设置预置的 业务处理
        :param value:
        :type value:
        :return:
        :rtype:
        """
        self._preset_usage = value

    def first(self):
        raise Exception(f"{self.name} is not ManyElements, Only ManyElements have")

    def index(self, i):
        raise Exception(f"{self.name} is not ManyElements, Only ManyElements have")

    @property
    def name(self):
        raise Exception("CdiscModel must defined name")

    @property
    def value(self):
        raise Exception(f"{self.name} is not ManyElements, ManyElements is not have this method")

    # TODO
    def each(self, usage, **attributes):
        raise Exception(f"{self.name} is not ManyElements, Only ManyElements have")

    # TODO
    def select(self, **attributes):
        raise Exception(f"{self.name} is not ManyElements, Only ManyElements have")

    def business(self):
        """
        执行预置的 业务处理
        :return:
        :rtype:
        """
        if self.preset_usage is not None:
            self.preset_usage.business(self)

    def usage(self, usage):
        """
        执行针对该节点的业务逻辑
        usage 为传入
        :param usage: 需要处理的业务逻辑
        :type usage: lambda, 函数，或 Usage 类型的实例
        :return:
        :rtype:
        """
        if callable(usage):  # lambda, 函数判断
            usage(self)
        elif isinstance(usage, Usage):  # 实例判断
            usage.business(self)

    def no_use(self):
        return self.name == "NoUse!"

    @property
    def parent(self):
        return self._parent

    @parent.setter
    def parent(self, value):
        self._parent = value
